"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultChainMethods = void 0;
const t = require("@babel/types");
const helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
exports.defaultChainMethods = [
    'trim',
    'join',
    'filter',
    'reduce',
    'forEach',
    'map',
    'some',
    'every',
    'find',
    'sort',
    'keys',
    'values',
    'entries',
];
exports.default = (0, helper_plugin_utils_1.declare)((api, options) => {
    const { chainMethods = exports.defaultChainMethods } = options;
    return {
        name: 'stats-plugin',
        visitor: {
            Program: {
                enter(_, state) {
                    state.currentPath = '';
                    state.stats = {};
                },
                exit(_, state) {
                    Object.keys(state.stats).forEach((key) => {
                        if (key.includes('.')) {
                            const newKey = key.split('.').reverse().join('.');
                            const value = state.stats[key];
                            delete state.stats[key];
                            state.stats[newKey] = value;
                        }
                    });
                    if (Object.keys(state.stats).length > 0) {
                        console.log(state.stats);
                    }
                },
            },
            CallExpression: {
                enter(path, state) {
                    if (t.isMemberExpression(path.node.callee) && t.isIdentifier(path.node.callee.property)) {
                        const methodName = path.node.callee.property.name;
                        if (chainMethods.includes(methodName)) {
                            if (methodName === 'join' && path.node.arguments.length > 1) {
                                return;
                            }
                            if (state.currentPath === '') {
                                state.currentPath = methodName;
                            }
                            else {
                                state.currentPath = state.currentPath + '.' + methodName;
                            }
                            if (!state.stats[state.currentPath]) {
                                state.stats[state.currentPath] = 1;
                            }
                            else {
                                state.stats[state.currentPath]++;
                            }
                            let next = path.node.callee.object;
                            while (t.isCallExpression(next) &&
                                t.isMemberExpression(next.callee) &&
                                t.isIdentifier(next.callee.property) &&
                                chainMethods.includes(next.callee.property.name)) {
                                state.currentPath = state.currentPath + '.' + next.callee.property.name;
                                if (!state.stats[state.currentPath]) {
                                    state.stats[state.currentPath] = 1;
                                }
                                else {
                                    state.stats[state.currentPath]++;
                                }
                                next = next.callee.object;
                            }
                            state.currentPath = '';
                        }
                    }
                    else {
                        state.currentPath = '';
                    }
                },
            },
        },
    };
});
